from bloonix.dbi import DBI
from bloonix.sql import SQL
from bloonix.logger import logger
from bloonix import utils

class Datastore(DBPartition):
    def __init__(self, **kwargs):
        self.log = logger("bloonix")
        self.dbi = DBI.setup(**kwargs)
        self.do = self.dbi.do
        self.ping = self.dbi.ping
        self.sql = SQL(driver=kwargs["driver"])

    def create_stats(self, data):
        stmt, bind = self.sql.insert(table="stat", data=data)
        self.dbi.do(stmt, bind)

    def create_results(self, data):
        stmt, bind = self.sql.insert(table="result", data=data)
        self.dbi.do(stmt, bind)

    def get_max_time_by_service(self, service_id):
        stmt, bind = self.sql.select(
            table="stat_hourly",
            columns="max(time) as max",
            condition=["where", ["service_id", service_id]])
        return self.dbi.fetchone(stmt, bind)["max"]

    def get_min_time_by_service(self, service_id):
        stmt, bind = self.sql.select(
            table="stat",
            columns="min(time) as min",
            condition=["where", ["service_id", service_id]])
        return self.dbi.fetchone(stmt, bind)["min"]

    def get_stats_for_archive(self, service_id, from_time, to_time):
        stmt, bind = self.sql.select(
            table="stat",
            columns="*",
            condition=[
                "where", ["service_id", service_id],
                "and", ["time", ">=", from_time],
                "and", ["time", "<=", to_time]
            ])
        return self.dbi.fetchall(stmt, bind)

    def create_archive_data(self, data):
        stmt, bind = self.sql.insert(
            table="stat_hourly",
            data=data)
        self.dbi.do(stmt, bind)

    def create_archive_partition(self, date):
        partitions = self.dbi.find_partitions("stat_hourly", r"stat_hourly_p\d+")
        m = re.search(r"^(\d\d\d\d)-(\d\d)", date)
        year = int(m.group(1))
        month = int(m.group(2))
        partition = "stat_hourly_p%s%02d" % (year, month)
        if partition in partitions:
            return
        from_time = utils.unixtime("%s-%02d-01 00:00:00" % (year, month))
        to_time = utils.unixtime("%s-%02d-01 00:00:00" % (year, month + 1))
        self.log.notice(
            "create range partition %s, range from %s to %s (%s - %s)" % (
                partition,
                utils.timestamp(from_time),
                utils.timestamp(to_time),
                from_time,
                to_time
            ))
        try:
            self.dbi.add_range_partition(
                table="stat_hourly",
                name=partition,
                from_value=from_time,
                to_value=to_time)
        except Exception:
            pass

    def get_archive_partitions(self):
        return self.dbi.find_partitions("stat_hourly", r"stat_hourly_p\d+")

    def delete_archive_partition(self, partition):
        self.dbi.delete_partition("stat_hourly", partition)

    def table_exist(self, table):
        stmt, bind = self.sql.select(
            table=table,
            columns="*",
            limit=1)
        try:
            self.dbi.fetchone(stmt, bind)
            return True
        except Exception:
            pass

    def connect(self):
        return self.dbi.connect()

    def disconnect(self):
        return self.dbi.disconnect()
