import re
from bloonix.dbi import partition_time

class DBPartition:
    def create_partition(self, table, days, index_on=None):
        partitions = self.dbi.find_partitions(table, r"%s_p\d+" % table)
        for i in range(days):
            t = partition_time.days_from_now(i)
            pname = "%s_p%s%s%s" % (table, t["year"], t["month"], t["day"])
            if pname in partitions:
                continue
            try:
                self.log.notice("create partition %s" % pname)
                self.dbi.add_range_partition(
                    table=table,
                    name=pname,
                    from_value=t["from"],
                    to_value=t["to"],
                    index_on=index_on)
            except Exception as e:
                self.log.warning(e)
        return True

    def delete_partition(self, table, keep_days):
        partitions = self.dbi.find_partitions(table, r"^%s_p\d{8}\Z" % table)
        for partition in partitions:
            f, t = partition_time.date_to_unixtime(partition)
            b = partition_time.days_back_to(keep_days)
            if t < b["to"]:
                self.log.notice("delete partition %s" % partition)
                self.dbi.delete_partition(table, partition)
