"""
In a multi-server setup, there must be one server that acts as the
primary server. All active servers are managed in this module.
"""
import re
import time
from bloonix.logger import logger

class Keepalived(CheckPrimary):
    def __init__(self, pm, queue, me, db):
        self.pm = pm
        self.queue = queue
        self.me = me
        self.db = db
        self.primary = None
        self.log = logger("bloonix")
        self.log.set_pattern("%Z", "keepalived")

    def run(self):
        while self.pm.done is False:
            try:
                self.db.connect()
                self.check_server()
                if self.am_i_primary():
                    if self.primary is None:
                        self.primary = True
                        self.log.notice("I am the primary server")
                else:
                    self.primary = None
            except Exception:
                self.log.exception()
            finally:
                self.db.disconnect()
            time.sleep(1)
        self.db.connect()
        self.db.delete_me_keepalived(self.me)
        self.db.disconnect()

    def check_server(self):
        self.db.delete_keepalived(15)
        self.db.update_me_keepalived(self.me, int(time.time()))
