import time
from bloonix import mstatus
from bloonix.logger import logger
import bloonix.utils as utils

class TimeoutChecker:
    def __init__(self, pm, queue, db, ds, config):
        self.pm = pm
        self.queue = queue
        self.db = db
        self.ds = ds
        self.config = config
        self.log = logger("bloonix")
        self.log.set_pattern("%Z", "timeout_checker")

    def run(self):
        while self.pm.done is False:
            try:
                self.pm.is_processing()
                self.check_queue()
                self.pm.is_waiting()
            except Exception:
                self.log.exception()
                self.log.set_pattern("%Y", "n/a")
            finally:
                self.pm.is_waiting()
            time.sleep(1)

    def check_queue(self):
        while self.pm.done is False:
            host_id = self.queue.pull(QUEUE_TIMED_OUT_HOSTS)
            if not host_id:
                break
            try:
                self.check_host(int(host_id))
            finally:
                self.db.disconnect()
                self.ds.disconnect()

    def check_host(self, host_id):
        self.log.notice("process host %s" % host_id)
        self.db.connect()
        services = self.db.get_services(
            host_id=host_id,
            with_vars=False,
            remote_check=[0,1],
            with_check_wtrm=True)

        unixtime = time.time()
        data = []
        for service_id, service in services.items():
            # new created service waiting for initialisation
            if service["force_check"] == 2:
                continue

            last_check = service["last_check"]
            if service["command"] == "check-wtrm":
                timeout = SERVICE_WTRM_TIMEOUT
            else:
                timeout = SERVICE_TIMEOUT

            if last_check + service["interval"] + timeout < unixtime:
                data.append({
                    "service_id": service["id"],
                    "data": {
                        "status": mstatus.CRITICAL,
                        "message": "SERVICE CHECK OVERDUE",
                    }
                })

        if not len(data):
            return

        host = self.db.get_host(host_id)
        self.log.set_pattern("%Y", host["id"])
        self.ds.connect()
        p = ProcessServiceData(
            db=self.db,
            ds=self.ds,
            host=self.db.get_host(host_id),
            config=self.config,
            unixtime=unixtime,
            update_last_check=False)
        p.run(data, services)
        self.log.set_pattern("%Y", "n/a")
