import re
import time
from bloonix.logger import logger
from bloonix.plugin import execute_check

class WebTransactionChecker:
    def __init__(self, pm, queue, db, ds, config):
        self.pm = pm
        self.queue = queue
        self.db = db
        self.ds = ds
        self.config = config
        self.log = logger("bloonix")
        self.log.set_pattern("%Z", "wtrm_checker")
        self.last_service_id = None

    def run(self):
        while self.pm.done is False:
            try:
                self.last_service_id = None
                self.read_queue()
            except Exception:
                self.log.exception()
            finally:
                self.finish()
            time.sleep(1)

    def read_queue(self):
        while self.pm.done is False:
            self.pm.is_waiting()
            pair = self.queue.pull(QUEUE_WTRM_READY_SERVICES)
            if not pair or not isinstance(pair, str):
                break

            self.pm.is_processing()
            m = re.search(r"^(\d+):(\d+)\Z", pair)
            if not m:
                self.log.error("invalid entry from queue QUEUE_WTRM_READY_SERVICES")
                continue

            host_id = int(m.group(1))
            service_id = self.last_service_id = int(m.group(2))

            self.db.connect()
            host = self.db.get_host(host_id)
            if not host:
                self.finish()
                continue

            s = ProcessGetServices(db=self.db, log=self.log)
            ret = s.get_wtrm_service(host, service_id)

            if not len(ret["services"]):
                self.finish()
                continue

            self.log.notice(
                "check host %s service %s command check-wtrm"
                % (host_id, service_id))

            service = ret["services"][0]

            data = execute_check(
                command=service["command"],
                command_options=service.get("command_options", None),
                plugin_path=self.config["plugin_path"],
                timeout=120,
                host_id=service["host_id"],
                service_id=service["service_id"])

            self.ds.connect()
            p = ProcessServiceData(
                db=self.db,
                ds=self.ds,
                host=host,
                config=self.config,
                unixtime=time.time())
            p.run([{"service_id": service_id, "data": data}])
            self.finish()

    def finish(self):
        self.db.disconnect()
        self.ds.disconnect()
        if self.last_service_id:
            service_id = self.last_service_id
            self.last_service_id = None
            try:
                self.queue.push(QUEUE_WTRM_FINISHED_SERVICES, service_id)
            except Exception:
                self.log.exception()
