import re
import time
from bloonix.logger import logger

class WebTransactionScheduler(CheckPrimary):
    def __init__(self, pm, queue, db, me):
        self.pm = pm
        self.queue = queue
        self.db = db
        self.me = me
        self.log = logger("bloonix")
        self.log.set_pattern("%Z", "wtrm_scheduler")
        self.services = {}
        self.last_service_update = 0
        self.primary = None

    def run(self):
        self.log.notice("up and running")
        while self.pm.done is False:
            try:
                self.db.connect()
                if self.am_i_primary():
                    if self.primary != self.me:
                        self.primary = self.me
                        self.services.clear()
                        self.log.notice("flush queue QUEUE_WTRM_READY_SERVICES")
                        self.queue.delete(QUEUE_WTRM_READY_SERVICES)
                        self.queue.delete(QUEUE_WTRM_FINISHED_SERVICES)
                    self.update_service_list()
                    self.manage_queue()
                else:
                    self.primary = None
                    self.services.clear()
            except Exception:
                self.log.exception()
            finally:
                self.db.disconnect()
            time.sleep(1)

    def update_service_list(self):
        # Fix time issues
        if self.last_service_update > time.time():
            self.last_service_update = time.time()
            return

        if self.last_service_update + 60 > time.time():
            return

        self.last_service_update = time.time()
        seen = {}
        for service in self.db.get_wtrm_services():
            service_id = service["id"]
            seen[service_id] = None
            if service["id"] not in self.services:
                self.services[service_id] = {
                    "host_id": service["host_id"],
                    "last_check": service["last_check"],
                    "queued": False}

            if service["retry_interval"] < 60:
                self.services[service_id]["poll_interval"] = service["retry_interval"]
            elif service["interval"] < 60:
                self.services[service_id]["poll_interval"] = service["interval"]
            else:
                # Force check must be taken into account
                self.services[service_id]["poll_interval"] = 60

        for service_id in list(self.services):
            if service_id not in seen:
                del self.services[service_id]

    def manage_queue(self):
        while True:
            service_id = self.queue.pull(QUEUE_WTRM_FINISHED_SERVICES)
            if not service_id or not re.search(r"^\d+\Z", service_id):
                break
            service_id = int(service_id)
            if service_id in self.services:
                self.log.info("wtrm service check finished %s" % service_id)
                self.services[service_id]["last_check"] = time.time()
                self.services[service_id]["queued"] = False

        for service_id, service in self.services.items():
            if service["queued"] != False:
                continue
            host_id = service["host_id"]
            if service["last_check"] + service["poll_interval"] < time.time():
                self.log.info("wtrm service check ready: %s" % service_id)
                self.queue.push(
                    QUEUE_WTRM_READY_SERVICES,
                    "%s:%s" % (host_id, service_id))
                service["queued"] = time.time()
